// KreisView.cpp: Implementierungsdatei
//

#include "stdafx.h"
#include "Beleg.h"

#include "BelegDoc.h"
#include "KreisView.h"

#include "DialogAuswahlKreis.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// KreisView

IMPLEMENT_DYNCREATE(KreisView, CFormView)

KreisView::KreisView()
	: CFormView(KreisView::IDD)
{
	//{{AFX_DATA_INIT(KreisView)
	m_Name = _T("");
	m_Radius = 0.0;
	m_X1 = 0.0;
	m_Y1 = 0.0;
	//}}AFX_DATA_INIT
}

KreisView::~KreisView()
{
}

void KreisView::DoDataExchange(CDataExchange* pDX)
{
	CFormView::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(KreisView)
	DDX_Control(pDX, IDC_COMBO_KOORD, m_Lokal_Koord);
	DDX_Text(pDX, IDC_EDIT_NAME, m_Name);
	DDX_Text(pDX, IDC_EDIT_RADIUS, m_Radius);
	DDV_MinMaxDouble(pDX, m_Radius, 1.e-004, 100000000000.);
	DDX_Text(pDX, IDC_EDIT_X1, m_X1);
	DDX_Text(pDX, IDC_EDIT_Y1, m_Y1);
	//}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP(KreisView, CFormView)
	//{{AFX_MSG_MAP(KreisView)
	ON_BN_CLICKED(IDC_BUTTON_AUSWAHL, OnButtonAuswahl)
	ON_BN_CLICKED(IDC_BUTTON_LOESCHEN, OnButtonLoeschen)
	ON_BN_CLICKED(IDC_BUTTON_NEU, OnButtonNeu)
	ON_BN_CLICKED(IDC_BUTTON_SPEICHERN, OnButtonSpeichern)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// Diagnose KreisView

#ifdef _DEBUG
void KreisView::AssertValid() const
{
	CFormView::AssertValid();
}

void KreisView::Dump(CDumpContext& dc) const
{
	CFormView::Dump(dc);
}

CBelegDoc* KreisView::GetDocument() // non-debug version is inline
{
	ASSERT(m_pDocument->IsKindOf(RUNTIME_CLASS(CBelegDoc)));
	return (CBelegDoc*)m_pDocument;
}
#endif //_DEBUG

/////////////////////////////////////////////////////////////////////////////
// Behandlungsroutinen fr Nachrichten KreisView 

void KreisView::setzeSubjektInView()
{
	if(subjekt)
	{
		m_Name = subjekt->GetName();
		m_X1 = subjekt->GetX1();
		m_Y1 = subjekt->GetY1();
		m_Radius = subjekt->GetRadius();
		UpdateData(FALSE);
		
		if(subjekt->GetLokal_Koord())
		{
			for(int i = 0; i < m_Lokal_Koord.GetCount(); i++)
			{
				if(subjekt->GetLokal_Koord() == (Lokal_Koord*)m_Lokal_Koord.GetItemDataPtr(i))
				{
					m_Lokal_Koord.SetCurSel(i);
					break;
				}
			}
		}
		else
		{
			int zeile = m_Lokal_Koord.FindStringExact(-1, "kein Koord");
			m_Lokal_Koord.SetCurSel(zeile);
		}

	}
}

void KreisView::leseSubjektAusView()
{
	if(subjekt)
	{
		UpdateData(TRUE);
		subjekt->SetName(m_Name);
		subjekt->SetX1(m_X1);
		subjekt->SetY1(m_Y1);
		subjekt->SetRadius(m_Radius);

		int zeile = m_Lokal_Koord.GetCurSel();
		Lokal_Koord* f= (Lokal_Koord*)m_Lokal_Koord.GetItemDataPtr(zeile);
		subjekt->SetLokal_Koord(f);
	}
}

void KreisView::OnButtonAuswahl() 
{
	DialogAuswahlKreis dlg;
	
	dlg.setArrayKreise(GetDocument()->getArrayKreise());
	
	if(dlg.DoModal() == IDOK)
	{
		subjekt = dlg.getSelKreis();
		setzeSubjektInView();
	}	
}

void KreisView::OnButtonLoeschen() 
{
	if(!neuesSubjekt)
	{
		GetDocument()->remove(subjekt);
		GetDocument()->UpdateAllViews(this, 1L, (CObject*)subjekt);
		OnButtonNeu();
	}
}

void KreisView::OnButtonNeu() 
{
	if(!neuesSubjekt)
	{
		subjekt = new Kreis;
		neuesSubjekt = TRUE;
		setzeSubjektInView();
	}	
}

void KreisView::OnButtonSpeichern() 
{
	if(subjekt)
	{
		leseSubjektAusView();
		if(neuesSubjekt)
		{
			GetDocument()->insert(subjekt);
			neuesSubjekt = FALSE;
		}
		else
			GetDocument()->update();

		GetDocument()->UpdateAllViews(this, 0L, (CObject*)subjekt->GetLokal_Koord());
	}
}

void KreisView::OnInitialUpdate() 
{
	CFormView::OnInitialUpdate();
	
	GetParentFrame()->RecalcLayout();
	ResizeParentToFit(FALSE);

	// ComboBox mit Firmen fllen

	m_Lokal_Koord.ResetContent();
	int zeile = m_Lokal_Koord.AddString("kein Koord");
	m_Lokal_Koord.SetItemDataPtr(zeile, (void*)NULL);
	for(int i = 0; i < GetDocument()->getArrayLokal_Koord()->GetSize(); i++)
	{
		Lokal_Koord* f = GetDocument()->getArrayLokal_Koord()->GetAt(i);
		int zeile = m_Lokal_Koord.AddString(f->GetName() );
		m_Lokal_Koord.SetItemDataPtr(zeile, (void*)f);
	}
	
	if(GetDocument()->getArrayKreise()->GetSize() < 1)
	{	// keine Verkufer im Array
		subjekt = new Kreis;
		neuesSubjekt = TRUE;
	}
	else
	{	// erste Firma als Subjekt
		subjekt = GetDocument()->getArrayKreise()->GetAt(0);
		neuesSubjekt = FALSE;
	}
	setzeSubjektInView();		
}

void KreisView::OnUpdate(CView* pSender, LPARAM lHint, CObject* pHint) 
{
	if(pHint && subjekt)
	{
		Lokal_Koord* f = static_cast<Lokal_Koord*>(pHint);
		if(f)
		{
			// Verkufer haben Firma in ComboBox m_firma
			int zeile = m_Lokal_Koord.GetCurSel();
			if(f == (Lokal_Koord*)m_Lokal_Koord.GetItemDataPtr(zeile) && lHint == 1L)
			{
				int zeile = m_Lokal_Koord.FindStringExact(-1, "kein Koord");
				m_Lokal_Koord.SetCurSel(zeile);
			}

			// alten Eintrag lschen
			for(int i = 0; i < m_Lokal_Koord.GetCount(); i++)
			{
				if((CObject*)m_Lokal_Koord.GetItemDataPtr(i) == pHint)
				{
					m_Lokal_Koord.DeleteString(i);
					break;
				}
			}
			// neuer Eintrag, wenn Hint auf 0
			if(lHint == 0L)
			{	
				int zeile = m_Lokal_Koord.AddString(f->GetName());
				m_Lokal_Koord.SetItemDataPtr(zeile, (void*)f);
			}
		}

		if((CObject*)subjekt == pHint)
			setzeSubjektInView();
	}
}

void KreisView::setSubjekt(CObject * neuerKreis)
{
	Kreis* tmp = dynamic_cast<Kreis*>(neuerKreis);
	if(tmp)
		subjekt = tmp;
}

CObject* KreisView::getSubjekt()
{
	return subjekt;
}
